Package: clock
Title: Date-Time Types and Tools
Version: 0.6.0
Authors@R: 
    c(person(given = "Davis",
             family = "Vaughan",
             role = c("aut", "cre"),
             email = "davis@rstudio.com"),
      person(given = "RStudio",
             role = c("cph", "fnd")))
Description: Provides a comprehensive library for date-time manipulations
    using a new family of orthogonal date-time classes (durations, time
    points, zoned-times, and calendars) that partition responsibilities so
    that the complexities of time zones are only considered when they are
    really needed. Capabilities include: date-time parsing, formatting,
    arithmetic, extraction and updating of components, and rounding.
License: MIT + file LICENSE
URL: https://clock.r-lib.org, https://github.com/r-lib/clock
BugReports: https://github.com/r-lib/clock/issues
Depends: R (>= 3.3)
Imports: ellipsis (>= 0.3.1), rlang (>= 0.4.10), tzdb (>= 0.2.0), vctrs
        (>= 0.3.7)
Suggests: covr, knitr, magrittr, pillar, rmarkdown, testthat (>=
        3.0.0), withr
LinkingTo: cpp11 (>= 0.4.0), tzdb (>= 0.2.0)
VignetteBuilder: knitr
Config/Needs/website: lubridate
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.1.2
SystemRequirements: C++11
NeedsCompilation: yes
Packaged: 2021-12-02 15:12:58 UTC; davis
Author: Davis Vaughan [aut, cre],
  RStudio [cph, fnd]
Maintainer: Davis Vaughan <davis@rstudio.com>
Repository: CRAN
Date/Publication: 2021-12-02 15:50:02 UTC
Built: R 4.0.5; x86_64-w64-mingw32; 2021-12-02 18:33:51 UTC; windows
Archs: x64
