test_that("codes are correct", {
  expect_identical(clock_months$january, 1L)
  expect_identical(clock_months$february, 2L)
  expect_identical(clock_months$march, 3L)
  expect_identical(clock_months$april, 4L)
  expect_identical(clock_months$may, 5L)
  expect_identical(clock_months$june, 6L)
  expect_identical(clock_months$july, 7L)
  expect_identical(clock_months$august, 8L)
  expect_identical(clock_months$september, 9L)
  expect_identical(clock_months$october, 10L)
  expect_identical(clock_months$november, 11L)
  expect_identical(clock_months$december, 12L)

  expect_identical(clock_weekdays$sunday, 1L)
  expect_identical(clock_weekdays$monday, 2L)
  expect_identical(clock_weekdays$tuesday, 3L)
  expect_identical(clock_weekdays$wednesday, 4L)
  expect_identical(clock_weekdays$thursday, 5L)
  expect_identical(clock_weekdays$friday, 6L)
  expect_identical(clock_weekdays$saturday, 7L)

  expect_identical(clock_iso_weekdays$monday, 1L)
  expect_identical(clock_iso_weekdays$tuesday, 2L)
  expect_identical(clock_iso_weekdays$wednesday, 3L)
  expect_identical(clock_iso_weekdays$thursday, 4L)
  expect_identical(clock_iso_weekdays$friday, 5L)
  expect_identical(clock_iso_weekdays$saturday, 6L)
  expect_identical(clock_iso_weekdays$sunday, 7L)
})
